/**
 * Copyright (c) 2021 Dominic Msters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "render.h"

primitive_t *primitive;

render_t * renderInit(char *name) {
  // Initialize the renderer
  render_t *render = malloc(sizeof(render_t));
  if(!render) return NULL; 
  // render->width = WINDOW_WIDTH_DEFAULT;
  // render->height = WINDOW_HEIGHT_DEFAULT;

  //GLEnable Things
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_DEPTH_TEST);
  glDepthMask(GL_TRUE);
  glDepthFunc(GL_LEQUAL);

  // Test
  for(int i = 0; i < 100; i++) {
    vertice_t *vertices = malloc(sizeof(vertice_t) * 4);
    indice_t *indices = malloc(sizeof(indice_t) * 6);

    vertices[0].x = -1, vertices[0].y = -1, vertices[0].z = 0;
    vertices[1].x =  1, vertices[1].y = -1, vertices[1].z = 0;
    vertices[2].x = -1, vertices[2].y =  1, vertices[2].z = 0;
    vertices[3].x =  1, vertices[3].y =  1, vertices[3].z = 0;

    indices[0] = (indice_t)0;
    indices[1] = (indice_t)1;
    indices[2] = (indice_t)2;
    
    indices[3] = (indice_t)1;
    indices[4] = (indice_t)2;
    indices[5] = (indice_t)3;

    primitive = primitiveCreate(4, 6);
    primitiveBufferVertices(primitive, 0, 4, vertices);
    primitiveBufferIndices(primitive, 0, 6, indices);
  }
  
  return render;
}

void renderFrame(render_t *render) {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  primitiveDraw(primitive, 0, 6);
}

bool renderDispose(render_t *render) {
  // Free up the renderer
  free(render);
  return true;
}

void renderSetResolution(render_t *render, uint32_t width, uint32_t height) {
  render->width = width;
  render->height = height;
  glViewport(0, 0, width, height);
}